@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,8 +47,9 @@
 #define APP_NAME		"powerstat"
 #define MIN_RUN_DURATION	(5*60)		/* We recommend a run of 5 minutes */
 #define SAMPLE_DELAY		(10)		/* Delay between samples in seconds */
-#define ROLLING_AVERAGE		(120)		/* 2 minute rolling average for power usage calculation */
-#define MAX_MEASUREMENTS 	(ROLLING_AVERAGE+10)
+#define ROLLING_AVERAGE_SECS	(120)		/* 2 minute rolling average for power usage calculation */
+#define STANDARD_AVERAGE_SECS	(120)
+#define MAX_MEASUREMENTS 	(ROLLING_AVERAGE_SECS + 10)
 #define START_DELAY		(3*60)		/* Delay to wait before sampling */
 #define MAX_PIDS		(32769)		/* Hash Max PIDs */
 #define	RATE_ZERO_LIMIT		(0.001)		/* Less than this we call the power rate zero */
@@ -77,13 +78,14 @@
 #define OPTS_REDO_WHEN_NOT_IDLE	(0x0004)	/* when idle below idle_threshold */
 #define OPTS_ZERO_RATE_ALLOW	(0x0008)	/* force allow zero rates */
 #define OPTS_ROOT_PRIV		(0x0010)	/* has root privilege */
+#define OPTS_STANDARD_AVERAGE	(0x0020)	/* calc standard average */
 
 #define OPTS_USE_NETLINK	(OPTS_SHOW_PROC_ACTIVITY | \
 				 OPTS_REDO_NETLINK_BUSY |  \
 				 OPTS_ROOT_PRIV)
 
-#define SYS_CLASS_POWER_SUPPLY	"/sys/class/power_supply"
-#define PROC_ACPI_BATTERY	"/proc/acpi/battery"
+#define SYS_CLASS_POWER_SUPPLY		"/sys/class/power_supply"
+#define PROC_ACPI_BATTERY		"/proc/acpi/battery"
 
 #define SYS_FIELD_VOLTAGE		"POWER_SUPPLY_VOLTAGE_NOW="
 #define SYS_FIELD_WATTS_RATE		"POWER_SUPPLY_POWER_NOW="
@@ -94,8 +96,8 @@
 
 /* Measurement entry */
 typedef struct {
-	double	value;
-	time_t	when;
+	double	value;	/* Measurment value */
+	time_t	when;	/* When it was measured */
 } measurement_t;
 
 /* Statistics entry */
@@ -126,13 +128,18 @@ static proc_info_t *proc_info[MAX_PIDS];	/* Proc hash table */
 static int max_readings;			/* number of samples to gather */
 static int sample_delay   = SAMPLE_DELAY;	/* time between each sample in secs */
 static int start_delay    = START_DELAY;	/* seconds before we start displaying stats */
-static int idle_threshold = IDLE_THRESHOLD;	/* lower than this and the CPU is busy */
+static double idle_threshold = IDLE_THRESHOLD;	/* lower than this and the CPU is busy */
 static log_t infolog;				/* log */
 static int opts;				/* opt arg opt flags */
 static volatile int stop_recv;			/* sighandler stop flag */
+static bool power_calc_from_capacity = false;	/* true of power is calculated via capacity change */
 
 
-char *file_get(const char *file)
+/*
+ *  file_get()
+ *	read a line from a /sys file
+ */
+static char *file_get(const char *const file)
 {
 	FILE *fp;
 	char buffer[4096];
@@ -145,7 +152,7 @@ char *file_get(const char *file)
 		return NULL;
 	}
 
-	fclose(fp);
+	(void)fclose(fp);
 
 	return strdup(buffer);
 }
@@ -174,13 +181,13 @@ static int tty_height(void)
  *  time_now()
  *	Gather current time in buffer
  */
-static void time_now(char *buffer, const size_t buflen)
+static void time_now(char *const buffer, const size_t buflen)
 {
 	struct tm tm;
 	time_t now;
 
-	time(&now);
-	localtime_r(&now, &tm);
+	(void)time(&now);
+	(void)localtime_r(&now, &tm);
 
 	snprintf(buffer, buflen, "%2.2d:%2.2d:%2.2d ",
 		tm.tm_hour, tm.tm_min, tm.tm_sec);
@@ -190,7 +197,7 @@ static void time_now(char *buffer, const size_t buflen)
  *  log_init()
  *	Initialise log head
  */
-static void log_init(void)
+static inline void log_init(void)
 {
 	infolog.head = NULL;
 	infolog.tail = NULL;
@@ -200,7 +207,7 @@ static void log_init(void)
  *  log_printf()
  *	append log messages in log list
  */
-static int log_printf(const char *fmt, ...)
+static int log_printf(const char *const fmt, ...)
 {
 	char buffer[4096];
 	char tmbuffer[10];
@@ -271,6 +278,7 @@ static void log_free(void)
  */
 static void handle_sigint(int dummy)
 {
+	(void)dummy;
 	stop_recv = 1;
 }
 
@@ -297,7 +305,7 @@ static int netlink_connect(void)
 
 	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
 		fprintf(stderr, "Bind failed: %s\n", strerror(errno));
-		close(sock);
+		(void)close(sock);
 		return -1;
 	}
 
@@ -340,7 +348,7 @@ static int netlink_listen(const int sock)
  *  stats_clear()
  *	clear stats
  */
-static void stats_clear(stats_t *stats)
+static void stats_clear(stats_t *const stats)
 {
 	int i;
 
@@ -354,7 +362,7 @@ static void stats_clear(stats_t *stats)
  *  stats_clear_all()
  *	zero stats data
  */
-static void stats_clear_all(stats_t *stats, const int n)
+static void stats_clear_all(stats_t *const stats, const int n)
 {
 	int i;
 
@@ -366,7 +374,7 @@ static void stats_clear_all(stats_t *stats, const int n)
  *  stats_read()
  *	gather pertinent /proc/stat data
  */
-static int stats_read(stats_t *info)
+static int stats_read(stats_t *const info)
 {
 	FILE *fp;
 	char buf[4096];
@@ -376,7 +384,7 @@ static int stats_read(stats_t *info)
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		if (strncmp(buf, "cpu ", 4) == 0)
-			sscanf(buf, "%*s %lf %lf %lf %lf %lf %lf %lf",
+			sscanf(buf, "%*s %15lf %15lf %15lf %15lf %15lf %15lf %15lf",
 				&(info->value[CPU_USER]),
 				&(info->value[CPU_NICE]),
 				&(info->value[CPU_SYS]),
@@ -385,15 +393,15 @@ static int stats_read(stats_t *info)
 				&(info->value[CPU_IRQ]),
 				&(info->value[CPU_SOFTIRQ]));
 		if (strncmp(buf, "ctxt ", 5) == 0)
-			sscanf(buf, "%*s %lf", &(info->value[CPU_CTXT]));
+			sscanf(buf, "%*s %15lf", &(info->value[CPU_CTXT]));
 		if (strncmp(buf, "intr ", 5) == 0)
-			sscanf(buf, "%*s %lf", &(info->value[CPU_INTR]));
+			sscanf(buf, "%*s %15lf", &(info->value[CPU_INTR]));
 		if (strncmp(buf, "procs_running ", 14) == 0)
-			sscanf(buf, "%*s %lf", &(info->value[CPU_PROCS_RUN]));
+			sscanf(buf, "%*s %15lf", &(info->value[CPU_PROCS_RUN]));
 		if (strncmp(buf, "procs_blocked ", 14) == 0)
-			sscanf(buf, "%*s %lf", &(info->value[CPU_PROCS_BLK]));
+			sscanf(buf, "%*s %15lf", &(info->value[CPU_PROCS_BLK]));
 	}
-	fclose(fp);
+	(void)fclose(fp);
 
 	return 0;
 }
@@ -409,7 +417,10 @@ static int stats_read(stats_t *info)
  *	gather up delta between last stats and current to get
  * 	some form of per sample accounting calculated.
  */
-static bool stats_gather(stats_t *s1, stats_t *s2, stats_t *res)
+static bool stats_gather(
+	const stats_t *const s1,
+	const stats_t *const s2,
+	stats_t *const res)
 {
 	double total;
 
@@ -426,7 +437,10 @@ static bool stats_gather(stats_t *s1, stats_t *s2, stats_t *res)
 	total = res->value[CPU_USER] + res->value[CPU_NICE] +
 		res->value[CPU_SYS] + res->value[CPU_IDLE] + res->value[CPU_IOWAIT];
 
-	/*  This should not happen, but we need to avoid division by zero or weird results */
+	/*
+	 * This should not happen, but we need to avoid division
+	 * by zero or weird results
+	 */
 	if (total <= 0.0)
 		return false;
 
@@ -473,7 +487,7 @@ static void stats_ruler(void)
  *
  *
  */
-static void row_increment(int *row)
+static void row_increment(int *const row)
 {
 	int tty_rows = tty_height();
 
@@ -488,7 +502,10 @@ static void row_increment(int *row)
  *  stats_print()
  *	print out statistics with accuracy depending if it's a summary or not
  */
-static void stats_print(const char *prefix, const bool summary, const stats_t *s)
+static void stats_print(
+	const char *const prefix,
+	const bool summary,
+	const stats_t *const s)
 {
 	char buf[10];
 
@@ -532,16 +549,15 @@ static void stats_print(const char *prefix, const bool summary, const stats_t *s
  *  stats_average_stddev_min_max()
  *	calculate average, std deviation, min and max
  */
-static void stats_average_stddev_min_max(stats_t *stats,
+static void stats_average_stddev_min_max(
+	const stats_t *const stats,
 	const int num,
-	stats_t *average,
-	stats_t *stddev,
-	stats_t *min,
-	stats_t *max)
+	stats_t *const average,
+	stats_t *const stddev,
+	stats_t *const min,
+	stats_t *const max)
 {
-	int i;
-	int j;
-	int valid;
+	int i, j, valid;
 
 	for (j = 0; j < MAX_VALUES; j++) {
 		double total = 0.0;
@@ -586,24 +602,130 @@ static void stats_average_stddev_min_max(stats_t *stats,
 	}
 }
 
+/*
+ *  calc_standard_average()
+ *	calculate a standard average based on first sample
+ *	and the current sample
+ */
+static void calc_standard_average(
+	double total_capacity,
+	double *const rate,
+	bool *const inaccurate)
+{
+	static time_t time_start = 0;
+	static double total_capacity_start = 0.0;
+	static bool first = true;
+	time_t time_now, dt;
+	double dw;
 
+	time_now = time(NULL);
+
+	if (first) {
+		time_start = time_now;
+		total_capacity_start = total_capacity;
+		first = false;
+		*rate = 0.0;
+		*inaccurate = true;
+		return;
+	}
+
+	dt = time_now - time_start;
+	dw = total_capacity_start - total_capacity;
+	if (dt <= 0 || dw <= 0.0) {
+		/* Something is wrong, can't be a good sample */
+		*rate = 0.0;
+		*inaccurate = true;
+		return;
+	}
+
+	*rate = 3600.0 * dw / dt;
+	/* Only after a fairly long duration can we be sure it is reasonable */
+	*inaccurate = dt < STANDARD_AVERAGE_SECS;
+}
+
+/*
+ *  calc_rolling_average()
+ *	calculate power by using rolling average
+ *
+ *  Battery is less helpful, we need to figure the power rate by looking
+ *  back in time, measuring capacity drop and figuring out the rate from
+ *  this.  We keep track of the rate over a sliding window of
+ *  ROLLING_AVERAGE_SECS seconds.
+ */
+static void calc_rolling_average(
+	double total_capacity,
+	double *const rate,
+	bool *const inaccurate)
+{
+	static int index = 0;
+	time_t time_now, dt;
+	static measurement_t measurements[MAX_MEASUREMENTS];
+	int i, j;
+
+	time_now = time(NULL);
+	measurements[index].value = total_capacity;
+	measurements[index].when  = time_now;
+	index = (index + 1) % MAX_MEASUREMENTS;
+	*rate = 0.0;
+
+	/*
+	 * Scan back in time for a sample that's > ROLLING_AVERAGE_SECS
+	 * seconds away and calculate power consumption based on this
+	 * value and interval
+	 */
+	for (j = index, i = 0; i < MAX_MEASUREMENTS; i++) {
+		j--;
+		if (j < 0)
+			j += MAX_MEASUREMENTS;
+
+		if (measurements[j].when) {
+			double dw = measurements[j].value - total_capacity;
+			dt = time_now - measurements[j].when;
+			*rate = 3600.0 * dw / dt;
+
+			if (time_now - measurements[j].when > ROLLING_AVERAGE_SECS) {
+				*inaccurate = false;
+				break;
+			}
+		}
+	}
+
+	/*
+	 *  We either have found a good measurement, or an estimate at this point, but
+	 *  is it valid?
+	 */
+	if (*rate < 0.0) {
+		*rate = 0.0;
+		*inaccurate = true;
+	}
+}
+
+
+static void calc_from_capacity(
+	double total_capacity,
+	double *const rate,
+	bool *const inaccurate)
+{
+	power_calc_from_capacity = true;
+
+	if (opts & OPTS_STANDARD_AVERAGE)
+		calc_standard_average(total_capacity, rate, inaccurate);
+	else
+		calc_rolling_average(total_capacity, rate, inaccurate);
+}
 
 /*
  *  power_rate_get_sys_fs()
  *	get power discharge rate from battery via /sys interface
  */
-static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccurate)
+static int power_rate_get_sys_fs(
+	double *const rate,
+	bool *const discharging,
+	bool *const inaccurate)
 {
 	DIR *dir;
 	struct dirent *dirent;
-	time_t time_now, dt;
-
-	static measurement_t measurements[MAX_MEASUREMENTS];
-	static int index = 0;
-	int i, j;
-	double total_watts = 0.0;
-	double total_capacity = 0.0;
-	double dw;
+	double total_watts = 0.0, total_capacity = 0.0;
 
 	*rate = 0.0;
 	*discharging = false;
@@ -622,14 +744,6 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 			int  val;
 			FILE *fp;
 
-			double voltage = 0.0;
-
-			double amps_rate = 0.0;
-			double amps_left = 0.0;
-
-			double watts_rate = 0.0;
-			double watts_left = 0.0;
-
 			/* Check that type field matches the expected type */
 			snprintf(path, sizeof(path), "%s/%s/type", SYS_CLASS_POWER_SUPPLY, dirent->d_name);
 			if ((data = file_get(path)) != NULL) {
@@ -647,37 +761,43 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 				return -1;
 			} else {
 				char buffer[4096];
+				double voltage = 0.0;
+				double amps_rate = 0.0;
+				double amps_left = 0.0;
+				double watts_rate = 0.0;
+				double watts_left = 0.0;
+
 				while (fgets(buffer, sizeof(buffer)-1, fp) != NULL) {
 					if (strstr(buffer, SYS_FIELD_STATUS_DISCHARGING))
 						*discharging = true;
 
 					if (strstr(buffer, SYS_FIELD_AMPS_LEFT) &&
 					    strlen(buffer) > sizeof(SYS_FIELD_AMPS_LEFT) - 1) {
-						sscanf(buffer + sizeof(SYS_FIELD_AMPS_LEFT) - 1, "%d", &val);
+						sscanf(buffer + sizeof(SYS_FIELD_AMPS_LEFT) - 1, "%12d", &val);
 						amps_left = (double)val / 1000000.0;
 					}
 
 					if (strstr(buffer, SYS_FIELD_WATTS_LEFT) &&
 					    strlen(buffer) > sizeof(SYS_FIELD_WATTS_LEFT) - 1) {
-						sscanf(buffer + sizeof(SYS_FIELD_WATTS_LEFT) - 1, "%d", &val);
+						sscanf(buffer + sizeof(SYS_FIELD_WATTS_LEFT) - 1, "%12d", &val);
 						watts_left = (double)val / 1000000.0;
 					}
 
 					if (strstr(buffer, SYS_FIELD_AMPS_RATE) &&
 					    strlen(buffer) > sizeof(SYS_FIELD_AMPS_RATE) - 1) {
-						sscanf(buffer + sizeof(SYS_FIELD_AMPS_RATE) - 1, "%d", &val);
+						sscanf(buffer + sizeof(SYS_FIELD_AMPS_RATE) - 1, "%12d", &val);
 						amps_rate = (double)val / 1000000.0;
 					}
 
 					if (strstr(buffer, SYS_FIELD_WATTS_RATE) &&
 					    strlen(buffer) > sizeof(SYS_FIELD_WATTS_RATE) - 1) {
-						sscanf(buffer + sizeof(SYS_FIELD_WATTS_RATE) - 1, "%d", &val);
+						sscanf(buffer + sizeof(SYS_FIELD_WATTS_RATE) - 1, "%12d", &val);
 						watts_rate = (double)val / 1000000.0;
 					}
 
 					if (strstr(buffer, SYS_FIELD_VOLTAGE) &&
 					    strlen(buffer) > sizeof(SYS_FIELD_VOLTAGE) - 1) {
-						sscanf(buffer + sizeof(SYS_FIELD_VOLTAGE) - 1, "%d", &val);
+						sscanf(buffer + sizeof(SYS_FIELD_VOLTAGE) - 1, "%12d", &val);
 						voltage = (double)val / 1000000.0;
 					}
 				}
@@ -688,10 +808,10 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 		}
 	} while (dirent);
 
-	closedir(dir);
+	(void)closedir(dir);
 
 	if (! *discharging) {
-		printf("Machine is not discharging and hence we cannot measure power usage.\n");
+		printf("Machine is not discharging, cannot measure power usage.\n");
 		return -1;
 	}
 
@@ -706,48 +826,8 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 		return 0;
 	}
 
-	/*
-	 *  Battery is less helpful, we need to figure the power rate by looking
-	 *  back in time, measuring capacity drop and figuring out the rate from
-	 *  this.  We keep track of the rate over a sliding window of ROLLING_AVERAGE
-	 *  seconds.
-	 */
-	time_now = time(NULL);
-
-	measurements[index].value = total_capacity;
-	measurements[index].when  = time_now;
-	index = (index + 1) % MAX_MEASUREMENTS;
-
-	/*
-	 * Scan back in time for a sample that's > ROLLING_AVERAGE seconds away
-	 * and calculate power consumption based on this value and interval
-	 */
-	for (j = index, i = 0; i < MAX_MEASUREMENTS; i++) {
-		j--;
-		if (j < 0)
-			j += MAX_MEASUREMENTS;
-
-		if (measurements[j].when) {
-			dw = measurements[j].value - total_capacity;
-			dt = time_now - measurements[j].when;
-			*rate = 3600.0 * dw / dt;
-
-			if (time_now - measurements[j].when > ROLLING_AVERAGE) {
-				*inaccurate = false;
-				break;
-			}
-		}
-	}
-
-	/*
-	 *  We either have found a good measurement, or an estimate at this point, but
-	 *  is it valid?
-	 */
-	if (*rate < 0.0) {
-		*rate = 0.0;
-		*inaccurate = true;
-	}
-
+	/*  Rate not known, so calculate it from historical data, sigh */
+	calc_from_capacity(total_capacity, rate, inaccurate);
 	return 0;
 }
 
@@ -755,20 +835,16 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
  *  power_rate_get_proc_acpi()
  *	get power discharge rate from battery via /proc/acpi interface
  */
-static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inaccurate)
+static int power_rate_get_proc_acpi(
+	double *const rate,
+	bool *const discharging,
+	bool *const inaccurate)
 {
 	DIR *dir;
 	FILE *file;
 	struct dirent *dirent;
 	char filename[PATH_MAX];
-	time_t time_now, dt;
-
-	static measurement_t measurements[MAX_MEASUREMENTS];
-	static int index = 0;
-	int i, j;
-	double total_watts = 0.0;
-	double total_capacity = 0.0;
-	double dw;
+	double total_watts = 0.0, total_capacity = 0.0;
 
 	*rate = 0.0;
 	*discharging = false;
@@ -829,12 +905,12 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 				}
 			}
 		}
-		fclose(file);
+		(void)fclose(file);
 
 		/*
-		 * Some HP firmware is broken and has an undefined 'present voltage'
-		 * field and instead returns this in the design_voltage field, so
-		 * bodge around this.
+		 * Some HP firmware is broken and has an undefined 
+		 * 'present voltage' field and instead returns this in
+		 * the design_voltage field, so work around this.
 		 */
 		if (voltage == 0.0) {
 			sprintf(filename, "/proc/acpi/battery/%s/info", dirent->d_name);
@@ -856,7 +932,7 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 		total_watts    += watts_rate + voltage * amps_rate;
 		total_capacity += watts_left + voltage * amps_left;
 	}
-	closedir(dir);
+	(void)closedir(dir);
 
 	if (! *discharging) {
 		printf("Machine is indicating it is not discharging and hence "
@@ -865,9 +941,10 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 	}
 
 	/*
- 	 *  If the battery is helpful it supplies the rate already, in which case
-	 *  we know the results from the battery are as good as we can and we don't
-	 *  have to figure out anything from capacity change over time.
+ 	 * If the battery is helpful it supplies the rate already, in which
+	 * case we know the results from the battery are as good as we can
+	 * and we don't have to figure out anything from capacity change over
+	 * time.
 	 */
 	if (total_watts > RATE_ZERO_LIMIT) {
 		*rate = total_watts;
@@ -875,51 +952,15 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 		return 0;
 	}
 
-	/*
-	 *  Battery is less helpful, we need to figure the power rate by looking
-	 *  back in time, measuring capacity drop and figuring out the rate from
-	 *  this.  We keep track of the rate over a sliding window of ROLLING_AVERAGE
-	 *  seconds.
-	 */
-	time_now = time(NULL);
-
-	measurements[index].value = total_capacity;
-	measurements[index].when  = time_now;
-	index = (index + 1) % MAX_MEASUREMENTS;
-
-	/*
-	 * Scan back in time for a sample that's > ROLLING_AVERAGE seconds away
-	 * and calculate power consumption based on this value and interval
-	 */
-	for (j = index, i = 0; i < MAX_MEASUREMENTS; i++) {
-		j--;
-		if (j < 0)
-			j += MAX_MEASUREMENTS;
-
-		if (measurements[j].when) {
-			dw = measurements[j].value - total_capacity;
-			dt = time_now - measurements[j].when;
-			*rate = 3600.0 * dw / dt;
-
-			if (time_now - measurements[j].when > ROLLING_AVERAGE) {
-				*inaccurate = false;
-				break;
-			}
-		}
-	}
-
-	/*
-	 *  We either have found a good measurement, or an estimate at this point, but
-	 *  is it valid?
-	 */
-	if (*rate < 0.0) {
-		*rate = 0.0;
-		*inaccurate = true;
-	}
+	/*  Rate not known, so calculate it from historical data, sigh */
+	calc_from_capacity(total_capacity, rate, inaccurate);
 	return 0;
 }
 
-static int power_rate_get(double *rate, bool *discharging, bool *inaccurate)
+static int power_rate_get(
+	double *const rate,
+	bool *const discharging,
+	bool *const inaccurate)
 {
 	struct stat buf;
 
@@ -939,7 +980,7 @@ static int power_rate_get(double *rate, bool *discharging, bool *inaccurate)
  *  proc_info_hash()
  * 	hash on PID
  */
-static int proc_info_hash(const pid_t pid)
+static inline int proc_info_hash(const pid_t pid)
 {
 	return pid % MAX_PIDS;
 }
@@ -948,7 +989,10 @@ static int proc_info_hash(const pid_t pid)
  *  proc_cmdline()
  *	get a processes cmdline text
  */
-static int proc_cmdline(const pid_t pid, char *cmdline, const size_t size)
+static int proc_cmdline(
+	const pid_t pid,
+	char *const cmdline,
+	const size_t size)
 {
 	FILE *fp;
 	char path[PATH_MAX];
@@ -958,7 +1002,7 @@ static int proc_cmdline(const pid_t pid, char *cmdline, const size_t size)
 	snprintf(path, sizeof(path), "/proc/%d/cmdline", pid);
 	if ((fp = fopen(path, "r")) != NULL) {
 		n = fread(cmdline, size, 1, fp);
-		fclose(fp);
+		(void)fclose(fp);
 	}
 	return n;
 }
@@ -969,8 +1013,7 @@ static int proc_cmdline(const pid_t pid, char *cmdline, const size_t size)
  */
 static char *proc_info_get(const pid_t pid)
 {
-	int i = proc_info_hash(pid);
-	int j;
+	int i = proc_info_hash(pid), j;
 
 	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
 		if ((proc_info[i] != NULL) && (proc_info[i]->pid == pid))
@@ -985,8 +1028,7 @@ static char *proc_info_get(const pid_t pid)
  */
 static void proc_info_free(const pid_t pid)
 {
-	int i = proc_info_hash(pid);
-	int j;
+	int i = proc_info_hash(pid), j;
 
 	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
 		if ((proc_info[i] != NULL) && (proc_info[i]->pid == pid)) {
@@ -1055,7 +1097,6 @@ static int proc_info_add(const pid_t pid)
 		return -1;
 	}
 	strcpy(info->cmdline, cmdline);
-
 	proc_info[i] = info;
 
 	return -1;
@@ -1078,7 +1119,7 @@ static int proc_info_load(void)
 			proc_info_add(atoi(dirent->d_name));
 	}
 
-	closedir(dir);
+	(void)closedir(dir);
 	return 0;
 }
 
@@ -1089,14 +1130,10 @@ static int proc_info_load(void)
 static int monitor(const int sock)
 {
 	ssize_t len;
+	int readings = 0, redone = 0, row = 0;
+	stats_t *stats, s1, s2, average, stddev, min, max;
 	struct nlmsghdr *nlmsghdr;
 	struct timeval t1, t2;
-	stats_t *stats;
-	stats_t s1, s2;
-	stats_t average, stddev, min, max;
-	int readings = 0;
-	int redone = 0;
-	int row = 0;
 
 	if ((stats = calloc(max_readings, sizeof(stats_t))) == NULL) {
 		fprintf(stderr, "Cannot allocate statistics table.\n");
@@ -1122,7 +1159,7 @@ static int monitor(const int sock)
 		struct timeval tv;
 		char __attribute__ ((aligned(NLMSG_ALIGNTO)))buf[4096];
 
-		gettimeofday(&t2, NULL);
+		(void)gettimeofday(&t2, NULL);
 		usec = ((t1.tv_sec + sample_delay - t2.tv_sec) * 1000000) + (t1.tv_usec - t2.tv_usec);
 		if (usec < 0)
 			goto sample_now;
@@ -1167,7 +1204,7 @@ sample_now:
 			}
 
 			time_now(tmbuffer, sizeof(tmbuffer));
-			gettimeofday(&t1, NULL);
+			(void)gettimeofday(&t1, NULL);
 			stats_read(&s2);
 
 			/*
@@ -1182,7 +1219,7 @@ sample_now:
 			}
 
 			if ((opts & OPTS_REDO_WHEN_NOT_IDLE) &&
-			    (stats[readings].value[CPU_IDLE] < (double)idle_threshold)) {
+			    (stats[readings].value[CPU_IDLE] < idle_threshold)) {
 				stats_clear(&stats[readings]);
 				stats_read(&s1);
 				gettimeofday(&t1, NULL);
@@ -1309,8 +1346,14 @@ sample_now:
 	printf("%6.2f Watts on Average with Standard Deviation %-6.2f\n",
 		average.value[POWER_RATE], stddev.value[POWER_RATE]);
 
-	free(stats);
+	if (power_calc_from_capacity) {
+		printf("Note: Power calculated from battery capacity drain, may not be accurate.\n");
+	} else {
+		if (opts & OPTS_STANDARD_AVERAGE)
+			printf("Note: The battery supplied suitable power data, -S option not required.\n");
+	}
 
+	free(stats);
 	return 0;
 }
 
@@ -1319,10 +1362,10 @@ sample_now:
  *  show_help()
  *	simple help
  */
-void show_help(char * const argv[])
+void show_help(char *const argv[])
 {
 	printf("%s, version %s\n\n", APP_NAME, VERSION);
-	printf("usage: %s [-b] [-d secs] [-h] [-i idle] [-p] [-r] [-s] [-z] [delay [count]]\n", argv[0]);
+	printf("usage: %s [-d secs] [-i idle] [-b|-h|-p|-r|-s|-z] [delay [count]]\n", argv[0]);
 	printf("\t-b redo a sample if a system is busy, considered less than %d%% CPU idle\n", IDLE_THRESHOLD);
 	printf("\t-d specify delay before starting, default is %d seconds\n", start_delay);
 	printf("\t-h show help\n");
@@ -1330,6 +1373,7 @@ void show_help(char * const argv[])
 	printf("\t-p redo a sample if we see process fork/exec/exit activity\n");
 	printf("\t-r redo a sample if busy and we see process activity (same as -b -p)\n");
 	printf("\t-s show process fork/exec/exit activity log\n");
+	printf("\t-S calculate power from capacity drain using standard average\n");
 	printf("\t-z forcibly ignore zero power rate stats from the battery\n");
 	printf("\tdelay: delay between each sample, default is %d seconds\n", SAMPLE_DELAY);
 	printf("\tcount: number of samples to take\n");
@@ -1337,19 +1381,15 @@ void show_help(char * const argv[])
 
 int main(int argc, char * const argv[])
 {
-    	int sock = -1;
 	double dummy_rate;
-	bool discharging;
-	bool dummy_inaccurate;
-	int ret = EXIT_FAILURE;
-	int i;
-	int run_duration;
+	int sock = -1, ret = EXIT_FAILURE, i, run_duration;
+	bool discharging, dummy_inaccurate;
 
     	signal(SIGINT, &handle_sigint);
     	siginterrupt(SIGINT, 1);
 
 	for (;;) {
-		int c = getopt(argc, argv, "bd:hi:prsz");
+		int c = getopt(argc, argv, "bd:hi:prszS");
 		if (c == -1)
 			break;
 		switch (c) {
@@ -1368,9 +1408,9 @@ int main(int argc, char * const argv[])
 			exit(EXIT_SUCCESS);
 		case 'i':
 			opts |= OPTS_REDO_WHEN_NOT_IDLE;
-			idle_threshold = atoi(optarg);
-			if ((idle_threshold < 0) || (idle_threshold > 99)) {
-				fprintf(stderr, "Idle threshold must be between 0..99\n");
+			idle_threshold = atof(optarg);
+			if ((idle_threshold < 0.0) || (idle_threshold > 99.99)) {
+				fprintf(stderr, "Idle threshold must be between 0..99.99\n");
 				exit(EXIT_FAILURE);
 			}
 			break;
@@ -1383,6 +1423,9 @@ int main(int argc, char * const argv[])
 		case 's':
 			opts |= OPTS_SHOW_PROC_ACTIVITY;
 			break;
+		case 'S':
+			opts |= OPTS_STANDARD_AVERAGE;
+			break;
 		case 'z':
 			opts |= OPTS_ZERO_RATE_ALLOW;
 			break;
